---------------------------------------------------------------------------------------------------------
--  ROCK OBJECT SCRIPT
---------------------------------------------------------------------------------------------------------
-- Creates a new 'rock' type of object.
--
-- Author:		GIANTS (bale.lua script)
-- Update:		akuenzi (akuenzi@gmail.com)
-- Date:		6/28/2016
--
-- Sincere thanks to Xentro for his patient mentorship in creating this script.
--

Rock = {};
Rock_mt = Class(Rock, MountableObject);

InitObjectClass(Rock, "Rock");

function Rock:new(isServer, isClient, customMt)
    local mt = customMt;
    if mt == nil then
        mt = Rock_mt;
    end;

    local self = MountableObject:new(isServer, isClient, mt);

    self.forcedClipDistance = 300;
    registerObjectClassName(self, "Rock");

    self.fillType = Fillable.FILLTYPE_ROCK;
    self.fillLevel = 0;
    self.rockValueScale = 1;

    g_currentMission:addLimitedObject(self);

    return self;
end;

function Rock:delete()
    if self.i3dFilename ~= nil then
        Utils.releaseSharedI3DFile(self.i3dFilename, nil, true);
    end
 
    g_currentMission:removeLimitedObject(self);
    unregisterObjectClassName(self);
    g_currentMission:removeItemToSave(self);
 
    Rock:superClass().delete(self);
end;

function Rock:readStream(streamId, connection)
    local i3dFilename = Utils.convertFromNetworkFilename(streamReadString(streamId));
 
    if self.nodeId == 0 then
        self:createNode(i3dFilename);
    end;
 
    self.fillLevel = streamReadFloat32(streamId);
    Rock:superClass().readStream(self, streamId, connection);
    g_currentMission:addItemToSave(self);
    
    self.rockValueScale = streamReadFloat32(streamId);
end;

function Rock:writeStream(streamId, connection)
    streamWriteString(streamId, Utils.convertToNetworkFilename(self.i3dFilename));
    streamWriteFloat32(streamId, self.fillLevel);
    Rock:superClass().writeStream(self, streamId, connection);
    
    streamWriteFloat32(streamId, self.rockValueScale);
end;

function Rock:update()
	Rock:superClass().update(self);
	if not self.firstRun then
		local _, y, _ = getWorldTranslation(self.nodeId)
		if y < -100 then
			self:delete()
		end;
		self.firstRun = true
	end;
end;

function Rock:updateTick(dt)
	Rock:superClass().updateTick(self, dt);
end;

function Rock:mount(object, node, x,y,z, rx,ry,rz)
    g_currentMission:removeItemToSave(self);
    Rock:superClass().mount(self, object, node, x,y,z, rx,ry,rz);
end;

function Rock:unmount()
    if Rock:superClass().unmount(self) then
        g_currentMission:addItemToSave(self);
        return true;
    end;
 
    return false;
end;

function Rock:setNodeId(nodeId)
    Rock:superClass().setNodeId(self, nodeId);

    self.fillLevel = 100;
    self.rockValueScale = Utils.getNoNil(tonumber(getUserAttribute(nodeId, "rockValueScale")), 1);

    self.fillType = Fillable.FILLTYPE_ROCK;
    local fillTypeStr = getUserAttribute(nodeId, "fillType");
    if fillTypeStr ~= nil then
        local fillTypeInt = Fillable.fillTypeNameToInt[fillTypeStr];
        if fillTypeInt ~= nil then
            self.fillType = fillTypeInt;
        end;
    end;
	self.forcedClipDistance = 300 * Utils.getProfileClassId();
end;

function Rock:createNode(i3dFilename)
    self.i3dFilename = i3dFilename;
    self.customEnvironment, self.baseDirectory = Utils.getModNameAndBaseDirectory(i3dFilename);
    local rockRoot = Utils.loadSharedI3DFile(i3dFilename);

    local rockId = getChildAt(rockRoot, 0);
    link(getRootNode(), rockId);
    delete(rockRoot);

    self:setNodeId(rockId);
end;

function Rock:load(i3dFilename, x,y,z, rx,ry,rz, fillLevel)
    self.i3dFilename = i3dFilename;
    self.customEnvironment, self.baseDirectory = Utils.getModNameAndBaseDirectory(i3dFilename);
    self:createNode(i3dFilename);
    setTranslation(self.nodeId, x, y, z);
    setRotation(self.nodeId, rx, ry, rz);
    if fillLevel ~= nil then
        self.fillLevel = fillLevel;
    end
    g_currentMission:addItemToSave(self);
end;

function Rock:loadFromMemory(nodeId, i3dFilename)
    self.i3dFilename = i3dFilename;
    self.customEnvironment, self.baseDirectory = Utils.getModNameAndBaseDirectory(i3dFilename);
    self:setNodeId(nodeId);
end;

function Rock:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

    local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#position"));
    local xRot,yRot,zRot = Utils.getVectorFromString(getXMLString(xmlFile, key.."#rotation"));
    if x == nil or y == nil or z == nil or xRot == nil or yRot == nil or zRot == nil then
        return false;
    end;
    local filename = getXMLString(xmlFile, key.."#filename");
    if filename == nil then
        return false;
    end;
    filename = Utils.convertFromNetworkFilename(filename);
    local rootNode = Utils.loadSharedI3DFile(filename);
    if rootNode == 0 then
        return false;
    end;

    local ret = false;
    local node = getChildAt(rootNode, 0);
    if node ~= nil and node ~= 0 then
        setTranslation(node, x,y,z);
        setRotation(node, xRot,yRot,zRot);
        link(getRootNode(), node);
        ret = true;
    end;
    delete(rootNode);
    if not ret then
        return false;
    end;

    self:loadFromMemory(node, filename);

    local fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
    if fillLevel ~= nil then
        self.fillLevel = fillLevel;
    end;
    
    local rockValueScale = Utils.getNoNil(tonumber(getXMLString(xmlFile, key.."#rockValueScale")), 1);
    if rockValueScale ~= nil then
        self.rockValueScale = rockValueScale;
    end

    return true;
end;

function Rock:getSaveAttributesAndNodes(nodeIdent)
    local x,y,z = getTranslation(self.nodeId);
	if y <= -100 then
		return nil, nil
	end;
    local xRot,yRot,zRot = getRotation(self.nodeId);
    local attributes = 'filename="'.. Utils.encodeToHTML(Utils.convertToNetworkFilename(self.i3dFilename))..'" position="'..x..' '..y..' '..z..'" rotation="'..xRot..' '..yRot..' '..zRot..'" rockValueScale="'..self.rockValueScale..'" fillLevel="'..self.fillLevel..'"';
    local nodes = "";
 
    return attributes, nodes;
end;

function Rock:getValue()
    local pricePerLiter = 0;
    local desc = Fillable.fillTypeIndexToDesc[self.fillType];
 
    if desc ~= nil then
        pricePerLiter = desc.pricePerLiter;
    end;
 
    return self.fillLevel * pricePerLiter * self.rockValueScale;
end;


---------------------------------------------------------------------------------------------------------
-- ENABLE BALE DESTROYER TRIGGERS TO BECOME ROCK DISPOSAL POINTS
---------------------------------------------------------------------------------------------------------

RockDispose = {}

function RockDispose:triggerCallback(superFunc, triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	if onEnter then
		if otherActorId ~= 0 then
			local object = g_currentMission:getNodeObject(otherActorId);
			if object ~= nil then
				if object:isa(Rock) then
                    playSample(g_currentMission.cashRegistrySound, 1, 1, 0);
	
					if g_currentMission:getIsServer() then

						local difficultyMultiplier = 2 ^ (3 - g_currentMission.missionStats.difficulty);
						local baseValue = object:getValue();
						g_currentMission:addSharedMoney(baseValue * difficultyMultiplier, "other");
						g_currentMission:addMoneyChange(baseValue * difficultyMultiplier, FSBaseMission.MONEY_TYPE_SINGLE, true);
						object:delete();
					end;
				end;
			end;
		end;
	end;
	
	superFunc(self, triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId);
end;

BaleDestroyerTrigger.triggerCallback = Utils.overwrittenFunction(BaleDestroyerTrigger.triggerCallback, RockDispose.triggerCallback);